@@ -8,13 +8,32 @@ from django.conf import settings
 from django.contrib.auth.models import User
 from sanza.Crm import models
 from sanza.Crm.widgets import CityAutoComplete, EntityAutoComplete, OpportunityAutoComplete, ContactAutoComplete
-from sanza.Crm.settings import get_default_country, NO_ENTITY_TYPE
+from sanza.Crm.settings import NO_ENTITY_TYPE
+from sanza.Crm.utils import get_default_country
 from datetime import datetime, date
 from form_utils.forms import BetterModelForm, BetterForm
 from djaloha.widgets import AlohaInput
 from django.utils import timezone
 from coop_cms.bs_forms import Form as BsForm, ModelForm as BsModelForm, BootstrapableMixin
 
+class BetterBsForm(BetterForm, BootstrapableMixin):
+    class Media:
+        css = {
+            'all': ('chosen/chosen.css',)
+        }
+        js = (
+            'chosen/chosen.jquery.js',
+        )
+        
+    def __init__(self, *args, **kwargs):
+        super(BetterBsForm, self).__init__(*args, **kwargs)
+        self._bs_patch_field_class()
+        for field in self.fields.values():
+            if field.widget.__class__.__name__ == forms.Select().__class__.__name__:
+                klass = field.widget.attrs.get("class", "") 
+                if not "chosen-select" in klass:
+                    field.widget.attrs["class"] = klass + " chosen-select"
+
 class BetterBsModelForm(BetterModelForm, BootstrapableMixin):
     class Media:
         css = {
@@ -145,16 +164,7 @@ class _CityBasedForm(object):
             except KeyError:
                 pass
         if not self.country_id:
-            cn = get_default_country()
-            try:
-                default_country = models.Zone.objects.get(name=cn, parent__isnull=True)
-            except models.Zone.DoesNotExist:
-                try:
-                    zt = models.ZoneType.objects.get(type="country")
-                except models.ZoneType.DoesNotExist:
-                    zt = models.ZoneType.objects.create(type="country", name=u"Country")
-                default_country = models.Zone.objects.create(name=cn, parent=None, type=zt)
-            self.country_id = default_country.id
+            self.country_id = get_default_country().id
             
         self.fields['city'].widget = CityAutoComplete(attrs={'placeholder': _(u'Enter a city'), 'size': '80'})
         
@@ -166,12 +176,11 @@ class _CityBasedForm(object):
         except:
             pass
     
-    def _get_country(self, id):
-        if id:
-            return models.Zone.objects.get(id=id, parent__isnull=True)
+    def _get_country(self, country_id):
+        if country_id:
+            return models.Zone.objects.get(id=country_id, parent__isnull=True, type__type="country")
         else:
-            cn = get_default_country()
-            return models.Zone.objects.get(name=cn, parent__isnull=True)
+            return get_default_country()
         
     def clean_city(self):
         city = self.cleaned_data['city']
@@ -193,8 +202,9 @@ class _CityBasedForm(object):
                     country_id = int(self.cleaned_data.get('country')) or self.country_id
                 except (ValueError, TypeError):
                     country_id = self.country_id
+                    
                 country = self._get_country(country_id)
-                default_country = models.Zone.objects.get(name=get_default_country(), parent__isnull=True)
+                default_country = get_default_country()
                 if country != default_country:
                     city, _is_new = models.City.objects.get_or_create(name=city, parent=country)
                 else:
@@ -215,13 +225,13 @@ class ModelFormWithCity(BetterBsModelForm, _CityBasedForm):
         super(ModelFormWithCity, self).__init__(*args, **kwargs)
         self._post_init(*args, **kwargs)
         
-class FormWithCity(BetterForm, _CityBasedForm):
+class FormWithCity(BetterBsForm, _CityBasedForm):
     country = forms.ChoiceField(required=False, label=_(u'Country'))
     zip_code = forms.CharField(required=False, label=_(u'zip code'))
     city = forms.CharField(required = False, label=_(u'City'))
     
     def __init__(self, *args, **kwargs):
-        super(BetterForm, self).__init__(*args, **kwargs)
+        super(FormWithCity, self).__init__(*args, **kwargs)
         self._post_init(*args, **kwargs)
     
 class EntityForm(ModelFormWithCity):
@@ -245,7 +255,7 @@ class EntityForm(ModelFormWithCity):
             ('address', {'fields': ['address', 'address2', 'address3', 'zip_code', 'city', 'cedex', 'country'], 'legend': _(u'Address')}),
             ('logo', {'fields': ['logo'], 'legend': _(u'Logo')}),
         ]
-        
+    
     def clean_logo(self):
         logo = self.cleaned_data["logo"]
         instance = self.instance
