@@ -6,10 +6,11 @@ from django.utils.translation import ugettext as _
 from sanza.Crm import models
 from django.core.exceptions import ValidationError
 from datetime import date, timedelta
-from sanza.Crm.settings import get_default_country
+#from sanza.Crm.settings import get_default_country
 from django.contrib.auth.models import User
-from django.db.models import Q
+from django.db.models import Q, Count
 from sanza.Crm.widgets import CityNoCountryAutoComplete, GroupAutoComplete
+from sanza.Crm.utils import get_default_country
 
 class EntityNameSearchForm(SearchFieldForm):
     _name = 'entity_name'
@@ -93,87 +94,172 @@ class ContactByModifiedDate(TwoDatesForm):
         dt2 += timedelta(1)
         return {'modified__gte': dt1, 'modified__lt': dt2, }
 
-class CitySearchForm(SearchFieldForm):
-    _name = 'city'
-    _label = _(u'City')
+class BaseCitySearchForm(SearchFieldForm):
     
     def __init__(self, *args, **kwargs):
-        super(CitySearchForm, self).__init__(*args, **kwargs)
+        super(BaseCitySearchForm, self).__init__(*args, **kwargs)
         qs = models.City.objects.order_by('name') 
         field = forms.ModelChoiceField(qs, label=self._label,
             widget = CityNoCountryAutoComplete(attrs={'placeholder': _(u'Enter a city'), 'size': '80'})
         )
         self._add_field(field)
-        
-    def get_lookup(self):
-        return Q(entity__city__id=self._value) | Q(city__id=self._value)
+
+class CitySearchForm(BaseCitySearchForm):
+    _name = 'city'
+    _label = _(u'City')
     
-class ZipCodeSearchForm(SearchFieldForm):
-    _name = 'zip_code'
-    _label = _(u'zip code')
+    def get_lookup(self):
+        return Q(city__id=self._value) | (Q(city__isnull=True) & Q(entity__city__id=self._value))
+    
+class EntityCitySearchForm(BaseCitySearchForm):
+    _name = 'entity_city'
+    _label = _(u'Entity city')
+
+    def get_lookup(self):
+        return Q(entity__city__id=self._value)
+
+class BaseZipCodeSearchForm(SearchFieldForm):
     
     def __init__(self, *args, **kwargs):
-        super(ZipCodeSearchForm, self).__init__(*args, **kwargs)
+        super(BaseZipCodeSearchForm, self).__init__(*args, **kwargs)
         field = forms.CharField(label=self._label,
             widget=forms.TextInput(attrs={'placeholder': _(u'Enter the beginning of the zip code')}))
         self._add_field(field)
-        
+    
+class ZipCodeSearchForm(BaseZipCodeSearchForm):
+    _name = 'zip_code'
+    _label = _(u'zip code')
+    
     def get_lookup(self):
-        return Q(entity__zip_code__istartswith=self._value) | Q(zip_code__istartswith=self._value)
+        return Q(zip_code__istartswith=self._value) | (Q(zip_code="") & Q(entity__zip_code__istartswith=self._value))
+
+
+class EntityZipCodeSearchForm(BaseZipCodeSearchForm):
+    _name = 'entity_zip_code'
+    _label = _(u'Entity zip code')
+            
+    def get_lookup(self):
+        return Q(entity__zip_code__istartswith=self._value)
         
 class ZoneSearchForm(SearchFieldForm):
+    
     def __init__(self, *args, **kwargs):
         super(ZoneSearchForm, self).__init__(*args, **kwargs)
-        qs = models.Zone.objects.filter(type__type=self._name).order_by('code', 'name') 
-        field = forms.ModelChoiceField(qs, label=self._label)
+        type_name = self._name.replace("entity_", "")
+        qs = models.Zone.objects.filter(type__type=type_name).order_by('code', 'name') 
+        field = forms.ModelChoiceField(qs, label=self._label, widget=self._get_widget())
         self._add_field(field)
         
+    def _get_widget(self):
+        return forms.Select(attrs={
+            'class': "chosen-select",
+            'data-placeholder': self._label,
+            'style': "width: 100%", 
+        })
+    
 class DepartmentSearchForm(ZoneSearchForm):
     _name = 'department'
     _label = _(u'Department')
         
     def get_lookup(self):
-        return Q(entity__city__parent__id=self._value) | Q(city__parent__id=self._value)
+        q1 = Q(city__parent__id=self._value) & Q(city__parent__type__type="department")
+        q2 = Q(city__isnull=True) & Q(entity__city__parent__id=self._value) & Q(entity__city__parent__type__type="department")
+        
+        return q1 | q2
+
+class EntityDepartmentSearchForm(ZoneSearchForm):
+    _name = 'entity_department'
+    _label = _(u'Entity Department')
+        
+    def get_queryset(self, qs):
+        return qs.filter(self.get_lookup()).filter(entity__city__parent__type__type="department")
+        
+    def get_lookup(self):
+        return Q(entity__city__parent__id=self._value)
         
 class RegionSearchForm(ZoneSearchForm):
     _name = 'region'
     _label = _(u'Region')
         
     def get_lookup(self):
-        return Q(entity__city__parent__parent__id=self._value) | Q(city__parent__parent__id=self._value)
+        q1 = Q(city__parent__parent__id=self._value) & Q(city__parent__parent__type__type="region")
+        q2 = Q(city__isnull=True) & Q(entity__city__parent__parent__id=self._value) & Q(entity__city__parent__parent__type__type="region")
+        return q1 | q2
+
+class EntityRegionSearchForm(ZoneSearchForm):
+    _name = 'entity_region'
+    _label = _(u'Entity Region')
+    
+    def get_queryset(self, qs):
+        return qs.filter(self.get_lookup()).filter(entity__city__parent__parent__type__type="region")
+        
+    def get_lookup(self):
+        return Q(entity__city__parent__parent__id=self._value)
         
 class CountrySearchForm(ZoneSearchForm):
     _name = 'country'
     _label = _(u'Country')
         
     def get_lookup(self):
-        default_country_name = get_default_country()
-        default_country = models.Zone.objects.get(name=default_country_name, type__type=self._name)
+        default_country = get_default_country()
         if int(self._value) == default_country.id:
-            return Q(entity__city__parent__type__type='department') | Q(city__parent__type__type='department')
+            return Q(city__parent__type__type='department') | (Q(city__isnull=True) & Q(entity__city__parent__type__type='department'))
         else:
-            return Q(entity__city__parent__id=self._value) | Q(city__parent__id=self._value)
+            return Q(city__parent__id=self._value) | (Q(city__isnull=True) & Q(entity__city__parent__id=self._value))
         
-class HasZipCodeForm(YesNoSearchFieldForm):
-    _name = 'has_zip'
-    _label = _(u'Has zip code?')
+class EntityCountrySearchForm(ZoneSearchForm):
+    _name = 'entity_country'
+    _label = _(u'Entity Country')
         
     def get_lookup(self):
-        no_zipcode = (Q(zip_code='') & Q(entity__zip_code=''))
-        if self.is_yes():
-            return ~no_zipcode
+        default_country = get_default_country()
+        
+        if int(self._value) == default_country.id:
+            return Q(entity__city__parent__type__type='department')
         else:
-            return no_zipcode
+            return Q(entity__city__parent__id=self._value, entity__city__parent__type__type="country")
+   
+
+class HasCityAndZipcodeForm(YesNoSearchFieldForm):
+    _name = 'has_city_and_zip'
+    _label = _(u'Has city and zip code?')
+        
+    def get_lookup(self):
+        contact_has_address = ~Q(zip_code='') & Q(city__isnull=False)
+        entity_has_address = ~Q(entity__zip_code='') & Q(entity__city__isnull=False)
+        has_address = contact_has_address | entity_has_address
+        if self.is_yes():
+            return has_address
+        else:
+            return ~has_address
             
 class ActionInProgressForm(YesNoSearchFieldForm):
     _name = 'action'
     _label = _(u'Action in progress')
+    
+    #def __init__(self, *args, **kwargs):
+    #    super(ActionInProgressForm, self).__init__(*args, **kwargs)
+    #    choices = ((1, _('Yes')),)
+    #    field = forms.ChoiceField(choices=choices, label=self._label)
+    #    self._add_field(field)
         
-    def get_lookup(self):
+    def get_queryset(self, qs):
+        qobjs = Q(entity__action__done=False) | Q(action__done=False)
         if self.is_yes():
-            return (Q(entity__action__done=False) | Q(action__done=False))
+            return qs.filter(qobjs)
         else:
-            return (Q(entity__action__done=True) | Q(action__done=True))
+            return qs.exclude(qobjs)
+
+class HasAction(YesNoSearchFieldForm):
+    _name = 'has_action'
+    _label = _(u'Has actions')
+        
+    def get_queryset(self, qs):
+        qs = qs.annotate(num_actions=Count('action'), num_entity_actions=Count('entity__action'))
+        if self.is_yes():
+            return qs.filter(Q(num_actions__gt=0) | Q(num_entity_actions__gt=0))
+        else:
+            return qs.filter().filter(Q(num_actions__eq=0) & Q(num_entity_actions__eq=0))
 
 # TODO ####################################        
 #class NoAction(YesNoSearchFieldForm):
@@ -565,24 +651,6 @@ class OpportunitySearchForm(SearchFieldForm):
     def get_lookup(self):
         return Q(action__opportunity__id=self._value) | Q(entity__action__opportunity__id=self._value)
         
-#class NotInOpportunitySearchForm(OpportunitySearchForm):
-#    _name = 'not_opportunity'
-#    _label = _(u'Not in opportunity')
-#    
-#    def get_lookup(self):
-#        return ~(Q(action__opportunity__id=self._value) | Q(entity__action__opportunity__id=self._value))
-#    
-#    #def get_exclude_lookup(self):
-#    #    return Q(action__opportunity__id=self._value) | Q(entity__action__opportunity__id=self._value)
-        
-#class ContactOpportunityStatusSearchForm(OpportunityStatusSearchForm):
-#    _name = 'contact_opportunity_status'
-#    _label = _(u'Opportunity status')
-#    
-#    def get_lookup(self):
-#        return {'action__opportunity__status': self._value,
-#                'action__opportunity__ended': False}
-
 class OpportunityNameSearchForm(SearchFieldForm):
     _name = 'opportunity_name'
     _label = _(u'Opportunity name')
@@ -595,197 +663,6 @@ class OpportunityNameSearchForm(SearchFieldForm):
         
     def get_lookup(self):
         return Q(action__opportunity__name__icontains=self._value) | Q(entity__action__opportunity__name__icontains=self._value)
-        
-#class ContactOpportunityNameSearchForm(OpportunityNameSearchForm):
-#    _name = 'opportunity_name'
-#    _label = _(u'Opportunity name')
-#        
-#    def get_lookup(self):
-#        return {'action__opportunity__name__icontains': self._value}
-
-#class NoOpportunityWithNameSearchForm(SearchFieldForm):
-#    _name = 'no_opportunity_with_name'
-#    _label = _(u'No opportunity with name')
-#    
-#    def __init__(self, *args, **kwargs):
-#        super(NoOpportunityWithNameSearchForm, self).__init__(*args, **kwargs)
-#        field = forms.CharField(label=self._label,
-#            widget=forms.TextInput(attrs={'placeholder': _(u'enter a part of the name of the searched opportunity')}))
-#        self._add_field(field)
-#        
-#    def get_lookup(self):
-#        pass
-#    
-#    def get_exclude_lookup(self):
-#        return {'action__opportunity__name__icontains': self._value}
-
-#class ContactNoOpportunityWithNameSearchForm(NoOpportunityWithNameSearchForm):
-#    _name = 'contact_no_opportunity_with_name'
-#    _label = _(u'Contact: No opportunity with name')
-#    
-#    def get_exclude_lookup(self):
-#        return {'action__opportunity__name__icontains': self._value}
-
-#class OpportunityByEndDate(TwoDatesForm):
-#    _name = 'opportunity_by_end_date'
-#    _label = _(u'Opportunity by end date')
-#    
-#    def get_lookup(self):
-#        dt1, dt2 = self._get_dates()
-#        return {'entity__opportunity__end_date__gte': dt1,
-#                'entity__opportunity__end_date__lte': dt2, }
-#
-#class ContactOpportunityByEndDate(OpportunityByEndDate):
-#    _name = 'contact_opportunity_by_end_date'
-#    _label = _(u'Contact: Opportunity by end date')
-#    
-#    def get_lookup(self):
-#        dt1, dt2 = self._get_dates()
-#        return {'action__opportunity__end_date__gte': dt1,
-#                'action__opportunity__end_date__lte': dt2, }
-#
-#class OpportunityByStartDate(TwoDatesForm):
-#    _name = 'opportunity_by_start_date'
-#    _label = _(u'Opportunity by start date')
-#    
-#    def get_lookup(self):
-#        dt1, dt2 = self._get_dates()
-#        return {'entity__opportunity__start_date__gte': dt1,
-#                'entity__opportunity__start_date__lte': dt2, }
-#
-#class ContactOpportunityByStartDate(OpportunityByStartDate):
-#    _name = 'contact_opportunity_by_start_date'
-#    _label = _(u'Contact: Opportunity by start date')
-#    
-#    def get_lookup(self):
-#        dt1, dt2 = self._get_dates()
-#        return {'action__opportunity__start_date__gte': dt1,
-#                'action__opportunity__start_date__lte': dt2, }
-#
-#class OpportunityBetween(TwoDatesForm):
-#    _name = 'opportunity_between'
-#    _label = _(u'Opportunity between')
-#    
-#    def get_lookup(self):
-#        dt1, dt2 = self._get_dates()
-#        q_obj1 = Q(entity__opportunity__end_date__gte=dt1) & Q(entity__opportunity__start_date__lte=dt2) \
-#            & Q(entity__opportunity__end_date__isnull=False) & Q(entity__opportunity__start_date__isnull=False)
-#        
-#        q_obj2 = Q(entity__opportunity__end_date__isnull=True) & Q(entity__opportunity__start_date__lte=dt2) \
-#            & Q(entity__opportunity__start_date__gte=dt1)
-#        
-#        q_obj3 = Q(entity__opportunity__end_date__isnull=True) & Q(entity__opportunity__ended=False) \
-#            & Q(entity__opportunity__start_date__lte=dt1)
-#        
-#        q_obj4 = Q(entity__opportunity__start_date__isnull=True) & Q(entity__opportunity__end_date__gte=dt1) \
-#            & Q(entity__opportunity__end_date__lte=dt2)
-#        
-#        return q_obj1 | q_obj2 | q_obj3 | q_obj4
-#    
-#class ContactOpportunityBetween(TwoDatesForm):
-#    _name = 'contact_opportunity_between'
-#    _label = _(u'Contact: Opportunity between')
-#    
-#    def get_lookup(self):
-#        dt1, dt2 = self._get_dates()
-#        q_obj1 = Q(action__opportunity__end_date__gte=dt1) & Q(action__opportunity__start_date__lte=dt2) \
-#            & Q(action__opportunity__end_date__isnull=False) & Q(action__opportunity__start_date__isnull=False)
-#        
-#        q_obj2 = Q(action__opportunity__end_date__isnull=True) & Q(action__opportunity__start_date__lte=dt2) \
-#            & Q(action__opportunity__start_date__gte=dt1)
-#        
-#        q_obj3 = Q(action__opportunity__end_date__isnull=True) & Q(action__opportunity__ended=False) \
-#            & Q(action__opportunity__start_date__lte=dt1)
-#        
-#        q_obj4 = Q(action__opportunity__start_date__isnull=True) & Q(action__opportunity__end_date__gte=dt1) \
-#            & Q(action__opportunity__end_date__lte=dt2)
-#        
-#        return q_obj1 | q_obj2 | q_obj3 | q_obj4
-#
-#    
-#class NoOpportunityBetween(OpportunityBetween):
-#    _name = 'no_opportunity_between'
-#    _label = _(u'No opportunity between')
-#    
-#    def get_lookup(self):
-#        q_obj = super(NoOpportunityBetween, self).get_lookup()
-#        return ~q_obj
-#
-#
-#class ContactNoOpportunityBetween(ContactOpportunityBetween):
-#    _name = 'contact_no_opportunity_between'
-#    _label = _(u'Contact: No opportunity between')
-#    
-#    def get_lookup(self):
-#        q_obj = super(ContactNoOpportunityBetween, self).get_lookup()
-#        return ~q_obj
-        
-#class OpportunityTypeSearchForm(SearchFieldForm):
-#    _name = 'opportunity_type'
-#    _label = _(u'Opportunity type')
-#    
-#    def __init__(self, *args, **kwargs):
-#        super(OpportunityTypeSearchForm, self).__init__(*args, **kwargs)
-#        qs = models.OpportunityType.objects.all()
-#        field = forms.ModelChoiceField(qs, label=self._label)
-#        self._add_field(field)
-#        
-#    def get_lookup(self):
-#        return {'action__opportunity__type': self._value}
-
-#class ContactOpportunityTypeSearchForm(OpportunityTypeSearchForm):
-#    _name = 'contact_opportunity_type'
-#    _label = _(u'Contact: Opportunity type')
-#    
-#    def get_lookup(self):
-#        return {'action__opportunity__type': self._value}
-
-#class NoOpportunityOfTypeSearchForm(SearchFieldForm):
-#    _name = 'no_opportunity_of_type'
-#    _label = _(u'No opportunity of type')
-#    
-#    def __init__(self, *args, **kwargs):
-#        super(NoOpportunityOfTypeSearchForm, self).__init__(*args, **kwargs)
-#        qs = models.OpportunityType.objects.all()
-#        field = forms.ModelChoiceField(qs, label=self._label)
-#        self._add_field(field)
-#    
-#    def get_lookup(self):
-#        pass
-#        
-#    def get_exclude_lookup(self):
-#        return {'action__opportunity__type': self._value}
-
-#class ContactNoOpportunityOfTypeSearchForm(NoOpportunityOfTypeSearchForm):
-#    _name = 'contact_no_opportunity_of_type'
-#    _label = _(u'Contact: No opportunity of type')
-#    
-#    def get_exclude_lookup(self):
-#        return {'action__opportunity__type': self._value}
-
-
-#class OpportunityInProgressForm(YesNoSearchFieldForm):
-#    _name = 'opportunity_in_progress'
-#    _label = _(u'Opportunity in progress')
-#    
-#    def get_lookup(self):
-#        if self.is_yes():
-#            return {'action__opportunity__ended': False}
-#    
-#    def get_exclude_lookup(self):
-#        return {'action__opportunity__ended': False}
-
-#class ContactOpportunityInProgressForm(OpportunityInProgressForm):
-#    _name = 'contact_opportunity_in_progress'
-#    _label = _(u'Contact: Opportunity in progress')
-#    
-#    def get_lookup(self):
-#        if self.is_yes():
-#            return {'action__opportunity__ended': False}
-#    
-#    def get_exclude_lookup(self):
-#        return {'action__opportunity__ended': False}
-
 
 class NoSameAsForm(YesNoSearchFieldForm):
     _name = 'no_same_as'
@@ -797,7 +674,7 @@ class NoSameAsForm(YesNoSearchFieldForm):
     def get_exclude_lookup(self):
         pass
     
-    def post_process(self, contacts):
+    def global_post_process(self, contacts):
         if self.is_yes():
             return contacts
         else:
@@ -974,4 +851,64 @@ class EntityWithCustomField(SearchFieldForm):
         value = int(self._value)
         cf = models.CustomField.objects.get(id=value, model=models.CustomField.MODEL_ENTITY)
         return Q(entity__entitycustomfieldvalue__custom_field=cf)
-    
\ No newline at end of file
+    
+class SortContacts(SearchFieldForm):
+    _name = 'sort'
+    _label = _(u'Sort contacts')
+    _contacts_display = True
+    
+    def __init__(self, *args, **kwargs):
+        super(SortContacts, self).__init__(*args, **kwargs)
+        choices = (
+            ('name', _(u'Name')),
+            ('entity', _(u'Entity')),
+            ('contact', _(u'Contact')),
+            ('zipcode', _(u'Zipcode')),
+        ) 
+        field = forms.CharField(
+            label=self._label,
+            widget=forms.Select(
+                choices=choices,
+                attrs={
+                    'class': "chosen-select",
+                    'style': "width: 100%",
+                }
+            )
+        )
+        self._add_field(field)
+        self.default_country = get_default_country()
+    
+    def _sort_by_name(self, c):
+        x = u"{0}".format(c.lastname, c.firstname) if c.entity.is_single_contact else c.entity.name
+        return x.upper()
+    
+    def _sort_by_contact(self, c):
+        x = u"{0} {1}".format(c.lastname, c.firstname)
+        return x.upper()
+    
+    def _sort_by_entity(self, c):
+        x = u"B" if c.entity.is_single_contact else u"A"
+        y = self._sort_by_name(c)
+        z = u"{0}".format(c.lastname, c.firstname) if not c.entity.is_single_contact else u""
+        return (x, y, z)
+    
+    def _sort_by_zipcode(self, c):
+        country = c.get_country()
+        x = c.get_zip_code or u'?'
+        city = c.get_city
+        if not city:
+            v = "C"
+            w = y = "?"
+        else:
+            v = u"A" if ((not country) or self.default_country.id == country.id) else u"B"
+            w = country.name if country else self.default_country.name
+            y = city.name
+        z = self._sort_by_name(c)
+        return (v, w, x, y, z)
+    
+    def get_queryset(self, qs):
+        return qs
+    
+    def global_post_process(self, contacts):
+        callback = getattr(self, '_sort_by_{0}'.format(self._value), None)
+        return sorted(contacts, key=callback)
