@@ -15,6 +15,9 @@ from django.core.mail import send_mail, EmailMessage
 from django.template.loader import get_template
 from models import ContactProfile
 from sanza.utils import now_rounded
+from registration.backends.default.views import RegistrationView, ActivationView
+from forms import UserRegistrationForm
+from utils import create_profile_contact, notify_registration
 
 @login_required 
 def edit_profile(request):
@@ -96,4 +99,39 @@ def post_message(request):
             'form': form,
         },
         context_instance=RequestContext(request)
-    )
\ No newline at end of file
+    )
+
+class AcceptNewsletterRegistrationView(RegistrationView):
+    
+    def get_form_class(self, request):
+        return UserRegistrationForm
+    
+    def register(self, request, **kwargs):
+        kwargs["username"] = kwargs["email"][:30]
+        user = super(AcceptNewsletterRegistrationView, self).register(request, **kwargs)
+        
+        #Store if 
+        user.contactprofile.accept_newsletter = kwargs.get('accept_newsletter', False)
+        user.contactprofile.accept_3rdparty = kwargs.get('accept_3rdparty', False)
+        
+        user.first_name = kwargs.get('firstname', "")
+        user.last_name = kwargs.get('lastname', "")
+        user.contactprofile.entity_type = kwargs.get('entity_type', None)
+        user.contactprofile.entity_name = kwargs.get('entity', "")
+        user.contactprofile.city = kwargs.get('city', None)
+        user.contactprofile.zip_code = kwargs.get('zip_code', None)
+        user.contactprofile.gender = kwargs.get('gender', 0) or 0
+        
+        user.save()
+        user.contactprofile.save()
+        
+        return user
+
+class AcceptNewsletterActivationView(ActivationView):
+    def activate(self, request, *args, **kwargs):
+        activated_user = super(AcceptNewsletterActivationView, self).activate(request, *args, **kwargs)
+        #The account has been activated: We can create the corresponding contact in Sanza
+        if activated_user:
+            profile = create_profile_contact(activated_user)
+            notify_registration(profile)
+        return activated_user
\ No newline at end of file
