@@ -22,6 +22,7 @@ from datetime import datetime
 from coop_cms.bs_forms import Form as BsForm, ModelForm as BsModelForm
 from django.template.loader import get_template
 from django.template import Context
+import json
 
 def load_from_name(constant_full_name):
     x = constant_full_name.split('.')
@@ -136,6 +137,7 @@ class SearchForm(forms.Form):
         self._forms = {}
         self._instance = instance
         self._save = save
+        self.contacts_display = False
         if instance: self.fields['name'].initial = instance.name
         if not data and instance:
             data = {}
@@ -152,7 +154,9 @@ class SearchForm(forms.Form):
                     if not self._forms.has_key(gr):
                         self._forms[gr] = []
                     form_class = get_field_form(field)
-                    self._forms[gr].append(form_class(gr, fid, {field: value}))
+                    form = form_class(gr, fid, {field: value})
+                    self.contacts_display = self.contacts_display or form._contacts_display
+                    self._forms[gr].append(form)
                 except ValueError:
                     pass
                 except Exception, msg:
@@ -232,48 +236,39 @@ class SearchForm(forms.Form):
         keys = self._forms.keys()
         keys.sort()
         contacts = set([])
-        post_processors = []
+        global_post_processors = []
+        self.contains_refuse_newsletter = False
+        
         for key in keys:
-            q_objs = []
-            exclude_q_objs = []
-            if 'secondary_contact' in [f._name for f in self._forms[key]]:
-                filter_lookup = {}
-            else:
-                filter_lookup = {'main_contact': True}
-            exclude_lookup = {}
+            post_processors = []
+            contacts_set = Contact.objects.all()
+            
+            if not ('secondary_contact' in [f._name for f in self._forms[key]]):
+                contacts_set = contacts_set.filter(main_contact=True)
+            
             for form in self._forms[key]:
-                lookup = form.get_lookup()
-                if lookup:
-                    if type(lookup) is dict:
-                        filter_lookup.update(lookup)
-                    elif type(lookup) is list:
-                        q_objs.extend(lookup)
-                    else:
-                        q_objs.append(lookup)
-
-                if hasattr(form, 'get_exclude_lookup'):
-                    xcl_lkp = form.get_exclude_lookup()
-                    if xcl_lkp:
-                        if type(xcl_lkp) is dict:
-                            exclude_lookup.update(xcl_lkp)
-                        elif type(xcl_lkp) is list:
-                            exclude_q_objs.extend(xcl_lkp)
-                        else:
-                            exclude_q_objs.append(xcl_lkp)
+                
+                contacts_set = form.get_queryset(contacts_set)
                 
                 if hasattr(form, 'post_process'):
                     post_processors.append(form.post_process)
+                    
+                if hasattr(form, 'global_post_process'):
+                    global_post_processors.append(form.global_post_process)
 
-            contacts_set = Contact.objects.filter(**filter_lookup)
-            for q_obj in q_objs:
-                contacts_set = contacts_set.filter(q_obj)
-            if exclude_lookup:
-                contacts_set = contacts_set.exclude(**exclude_lookup)
-            for q_obj in exclude_q_objs:
-                contacts_set = contacts_set.exclude(q_obj)
-            contacts = contacts.union(set(contacts_set))
             for pp in post_processors:
-                contacts = pp(contacts)
+                contacts_set = pp(contacts_set)
+                
+            contacts = contacts.union(set(contacts_set))
+        
+        for gpp in global_post_processors:
+            contacts = gpp(contacts)
+        
+        for c in contacts:
+            if not c.accept_newsletter:
+                self.contains_refuse_newsletter = True
+                break
+                
         return list(contacts)
     
     def _get_filter_func(self):
@@ -283,7 +278,6 @@ class SearchForm(forms.Form):
         return lambda c: c and (not c.has_left)
 
     def get_contacts_by_entity(self):
-        self.contains_refuse_newsletter = False
         contacts = self._get_contacts()
         contacts_count = 0
         entities = {}
@@ -296,8 +290,6 @@ class SearchForm(forms.Form):
                 entities[entity.id] = (entity, [])
                 empty_entities[entity.id] = entity
             if pass_filter:
-                if not contact.accept_newsletter:
-                    self.contains_refuse_newsletter = True
                 entities[entity.id][1].append(contact)
                 entities[entity.id][1].sort(key=lambda c: c.lastname.lower())
                 empty_entities.pop(entity.id, None)
@@ -355,6 +347,8 @@ class SearchForm(forms.Form):
         return html
 
 class SearchFieldForm(BsForm):
+    _contacts_display = False
+    
     def __init__(self, block, count, data=None, *args, **kwargs):
         self._block = block
         self._count = count
@@ -384,6 +378,28 @@ class SearchFieldForm(BsForm):
     def as_it_is(self):
         t = get_template("Search/_search_field_form.html")
         return t.render(Context({"form": self}))
+    
+    def get_queryset(self, qs):
+        if hasattr(self, 'get_lookup'):
+            lookup = self.get_lookup()
+            if lookup:
+                if type(lookup) is dict:
+                    qs = qs.filter(**lookup)
+                elif type(lookup) is list:
+                    qs = qs.filter(*lookup)
+                else:
+                    qs = qs.filter(lookup)
+                    
+        if hasattr(self, 'get_exclude_lookup'):
+            lookup = self.get_exclude_lookup()
+            if lookup:
+                if type(lookup) is dict:
+                    qs = qs.exclude(**lookup)
+                elif type(lookup) is list:
+                    qs = qs.exclude(*lookup)
+                else:
+                    qs = qs.exclude(lookup)
+        return qs
         
 class TwoDatesForm(SearchFieldForm):
     
@@ -434,7 +450,15 @@ class SearchActionBaseForm:
 
     def get_contacts(self):
         ids = self.cleaned_data["contacts"].split(";")
-        return Contact.objects.filter(id__in=ids)
+        contacts = Contact.objects.filter(id__in=ids)
+        #contact_by_ids = {unicode(c.id): c for c in contacts}
+        contact_by_ids = {}
+        for c in contacts:
+            contact_by_ids[unicode(c.id)] = c
+        ordered_contacts = []
+        for cid in ids:
+            ordered_contacts.append(contact_by_ids[cid])
+        return ordered_contacts
 
 
 class SearchActionForm(forms.Form, SearchActionBaseForm):
@@ -560,4 +584,35 @@ class GroupForContactsForm(forms.Form):
         return Contact.objects.filter(id__in=ids)
 
 class SearchNameForm(forms.Form):
+    search_id = forms.IntegerField(required=False, widget=forms.HiddenInput())
     name = forms.CharField(required=True, label=_(u"Name"))
+    search_fields = forms.CharField(required=True, widget=forms.HiddenInput())
+    
+    def clean_name(self):
+        search_id = self.cleaned_data["search_id"]
+        
+        name = self.cleaned_data["name"]
+        if not name:
+            raise ValidationError(_(u"This field is required"))
+        
+        qs = models.Search.objects.filter(name=name)
+        if search_id:
+            qs = qs.exclude(id=search_id)
+        if qs.count()>0:
+            raise ValidationError(_(u"This name is already used"))
+        
+        if search_id:
+            search = models.Search.objects.get(id=search_id)
+            if search.name != name:
+                search.name = name
+                search.save()
+        else:
+            search = models.Search(name=name)        
+        return search
+    
+    def clean_search_fields(self):
+        search_fields = self.cleaned_data["search_fields"]
+        data = json.loads(search_fields)
+        for k in ('csrfmiddlewaretoken', 'excluded', 'name', 'field_choice'):
+            data.pop(k, None)
+        return data
\ No newline at end of file
