@@ -97,7 +97,7 @@ def quick_search(request):
 @user_passes_test(can_access)
 def search(request, search_id=0, group_id=0, opportunity_id=0, city_id=0):
     message = ''
-    entities = []
+    results = []
     search=None
     field_choice_form = forms.FieldChoiceForm()
     contains_refuse_newsletter = False
@@ -105,6 +105,7 @@ def search(request, search_id=0, group_id=0, opportunity_id=0, city_id=0):
     contacts_count = 0
     has_empty_entities = False
     group = opportunity = city = None
+    contacts_display = False
     
     if request.method == "POST":
         data = request.POST
@@ -121,46 +122,64 @@ def search(request, search_id=0, group_id=0, opportunity_id=0, city_id=0):
     if data:
         search_form = forms.SearchForm(data)
         if search_form.is_valid():
-            entities, contacts_count, has_empty_entities = search_form.get_contacts_by_entity()
-            contains_refuse_newsletter = search_form.contains_refuse_newsletter
-            if not entities:
+            contacts_display = search_form.contacts_display
+            
+            if not contacts_display:
+                results, contacts_count, has_empty_entities = search_form.get_contacts_by_entity()
+                contains_refuse_newsletter = search_form.contains_refuse_newsletter
+            else:
+                results = search_form.get_contacts()
+                contacts_count, has_empty_entities = len(results), False
+                contains_refuse_newsletter = search_form.contains_refuse_newsletter
+            
+            if not results:
                 message = _(u'Sorry, no results found')
     else:
         search = get_object_or_404(models.Search, id=search_id) if search_id else None
         search_form = forms.SearchForm(instance=search)
     
-    entities_count = len(entities)
+    entities_count = 0 if contacts_display else len(results)
     return render_to_response(
         'Search/search.html',
         {
-            'request': request, 'entities': entities, 'nb_entities_by_page': getattr(settings, 'SANZA_SEARCH_NB_IN_PAGE', 50),
+            'request': request, 'results': results, 'nb_results_by_page': getattr(settings, 'SANZA_SEARCH_NB_IN_PAGE', 50),
             'field_choice_form': field_choice_form, 'message': message, 'has_empty_entities': has_empty_entities,
             'search_form': search_form, 'search': search, 'contacts_count': contacts_count, 'entities_count': entities_count,
             'contains_refuse_newsletter': contains_refuse_newsletter, 'group': group, 'opportunity': opportunity,
-            'city': city,
+            'city': city, 'contacts_display': contacts_display,
         },
         context_instance=RequestContext(request)
     )
 
 @user_passes_test(can_access)
-def search_name(request, search_id=0):
+@popup_redirect
+def save_search(request, search_id):
+    try:
+        search_id = int(search_id)
+        if search_id:
+            search = get_object_or_404(models.Search, id=search_id)
+        else:
+            search = models.Search()
+    except ValueError:
+        raise Http404
+    
     if request.method == "POST":
         form = forms.SearchNameForm(request.POST)
         if form.is_valid():
-            save_url = reverse("search_save", args=[search_id])
-            script = u'''
-                $("input[name=name]").val("{0}")
-                $("form.search-form").attr('action', "{1}");
-                $("form.search-form").submit();
-                $("form.search-form").attr('action', '');
-                $.colorbox.close();
-            '''.format(form.cleaned_data["name"], save_url)
-            return HttpResponse(u"<script>{0}</script>".format(script))
+            search = form.cleaned_data['name']
+            search_fields = form.cleaned_data['search_fields']
+            search_fields['name'] = search.name
+            search_form = forms.SearchForm(search_fields, instance=search, save=True)
+            if search_form.is_valid():
+                search_form.save_search()
+                return HttpResponseRedirect(reverse('search', args=[search.id])) 
     else:
         initial = {}
         if search_id:
             search = get_object_or_404(models.Search, id=search_id)
-            initial = {'name': search.name}
+            initial = {'name': search.name, 'search_id': search_id}
+        else:
+            initial = {'search_id': 0}
         form = forms.SearchNameForm(initial=initial)
     
     return render_to_response(
@@ -172,20 +191,6 @@ def search_name(request, search_id=0):
         context_instance=RequestContext(request)
     )
 
-@user_passes_test(can_access)
-def save_search(request, search_id=0):
-    if search_id:
-        search = get_object_or_404(models.Search, id=search_id)
-    else:
-        search = models.Search()
-    
-    if request.method == "POST":
-        search_form = forms.SearchForm(request.POST, instance=search, save=True)
-        if search_form.is_valid():
-            search_form.save_search()
-            return HttpResponseRedirect(reverse('search', args=[search.id]))    
-    return HttpResponseRedirect(reverse('search'))
-    
 @user_passes_test(can_access)
 def get_field(request, name):
     block = request.GET.get('block')
@@ -302,7 +307,9 @@ def export_contacts_as_excel(request):
         search_form = forms.SearchForm(request.POST)
         if search_form.is_valid():
             contacts = search_form.get_contacts()
-            contacts.sort(key=lambda x: u"{0}-{1}-{2}".format(x.entity, x.lastname, x.firstname))
+            if not search_form.contacts_display:
+                #if the form alread has a sort criteria: don't sort again
+                contacts.sort(key=lambda x: u"{0}-{1}-{2}".format(x.entity, x.lastname, x.firstname))
             
             #create the excel document
             wb = xlwt.Workbook()
@@ -532,7 +539,6 @@ def export_to_pdf(request):
                 if form.is_valid():
                     template_name = form.cleaned_data['template']
                     contacts = form.get_contacts()
-                    
                     context = {
                         "contacts": contacts,
                         "search_dict": json.loads(form.cleaned_data['search_dict']),
@@ -560,7 +566,6 @@ def export_to_pdf(request):
                 search_form = forms.SearchForm(request.POST)
                 if search_form.is_valid():
                     contacts = search_form.get_contacts()
-                    
                     search_dict = json.dumps(search_form.serialize())
                     form = PdfTemplateForm(initial={'contacts': contacts, 'search_dict': search_dict})
                     return render_to_response(
